/* -*- C++ -*-
 * $Id: flbrowser.cc,v 1.42 2006/12/29 04:45:14 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flbrowser.h"
#include "flenum.h"
#include "flcolor.h"

#define DEF_BROWSER_CONST_NUM(c) \
  rb_define_const(rb_cFLBrowser_,#c,INT2NUM(Fl_Browser_::c))

VALUE rb_cFLBrowser_;
VALUE rb_cFLBrowser;
VALUE rb_cFLHold_Browser;
VALUE rb_cFLMulti_Browser;
VALUE rb_cFLSelect_Browser;
#ifdef HAVE_FL_FL_CHECK_BROWSER_H
VALUE rb_cFLCheck_Browser;
#endif

static int no_columns[1] = {0};

static FN_S_NEW(Browser_);
WIDGET_FNS(Browser_);
static FN_GET_INT(Browser_, deselect);
static FN_GETSET_INT(Browser_, position);
static FN_GET_VALUE(Browser_, selection);
static FN_GET_VALUE_SET_INT(Browser_, find_item);
static FN_SET_VALUE(Browser_, deleting);
static FN_SET_VALUE2(Browser_, inserting);
static FN_SET_VALUE2(Browser_, replacing);
static FN_SET_INT(Browser_, has_scrollbar);
static FN_GETSET_INT(Browser_, hposition);
static FN_GETSET_COLOR(Browser_, textcolor);
static FN_GETSET_ENUM(Browser_, Fl_Font, textfont);
static FN_GETSET_INT(Browser_, textsize);
static FN_GET_VALUE(Browser_, top);
static FN_CALL(Browser_, new_list);
static FN_SET_VALUE(Browser_, redraw_line);
static FN_CALL(Browser_, redraw_lines);
static FN_CALL(Browser_, scrollbar_left);
static FN_CALL(Browser_, scrollbar_right);
static FN_GET_INT(Browser_, leftedge);

static VALUE rb_FLBrowser__bbox(VALUE self)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  int x,y,w,h;
  widget->bbox(x, y, w, h);
  return rb_ary_new3(4, INT2NUM(x), INT2NUM(y), INT2NUM(w), INT2NUM(h));
}

static VALUE rb_FLBrowser__item_quick_height(VALUE self, VALUE item)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  return INT2NUM(widget->call_parent_item_quick_height(VAL2CPTR(item)));
}

static VALUE rb_FLBrowser__incr_height(VALUE self)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  return INT2NUM(widget->call_parent_incr_height());
}

static VALUE rb_FLBrowser__full_height(VALUE self)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  return INT2NUM(widget->call_parent_full_height());
}

static VALUE rb_FLBrowser__full_width(VALUE self)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  return INT2NUM(widget->call_parent_full_width());
}

static VALUE rb_FLBrowser__item_select(int argc, VALUE argv[], VALUE self)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  VALUE item,state;

  switch( rb_scan_args(argc, argv, "11", &item, &state) ){
  case 1:
    widget->call_parent_item_select(VAL2CPTR(item));
    break;
  case 2:
    widget->call_parent_item_select(VAL2CPTR(item), BOOL2INT(state));
    break;
  };
  return Qnil;
}

static VALUE rb_FLBrowser__select(int argc, VALUE argv[], VALUE self)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  VALUE item, state, docb;
  int r;

  switch( rb_scan_args(argc, argv, "12", &item, &state, &docb) ){
  case 1:
    r = widget->select(VAL2CPTR(item));
    break;
  case 2:
    r = widget->select(VAL2CPTR(item), FIX2INT(state));
    break;
  case 3:
    r = widget->select(VAL2CPTR(item), FIX2INT(state), BOOL2INT(docb));
    break;
  };
  return INT2BOOL(r);
}

static VALUE rb_FLBrowser__select_only(int argc, VALUE argv[], VALUE self)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  VALUE item, docb;
  int r;

  switch( rb_scan_args(argc, argv, "11", &item, &docb) ){
  case 1:
    r = widget->select_only(VAL2CPTR(item));
    break;
  case 2:
    r = widget->select_only(VAL2CPTR(item), BOOL2INT(docb));
    break;
  };
  return INT2BOOL(r);
}

static VALUE rb_FLBrowser__item_selected(VALUE self, VALUE item)
{
  RBFLBrowser_ *widget = (RBFLBrowser_*)rb_to_fl(self);
  VALUE r;

  r = widget->call_parent_item_selected(VAL2CPTR(item));
  return INT2BOOL(r);
}


static FN_S_NEW(Browser);
WIDGET_FNS(Browser);
static FN_GET_INT(Browser,size);
static FN_INIT2(Browser, Fl_Group::size, {
  // replace the default value with our default value -- no_columns.
  widget->column_widths(no_columns);
  // Fltk::Browser has @items for some methods such as `add',`remove' and `clear'.
  rb_ivar_set(self, rb_id_iitems, rb_ary_new());
});
static FN_SET_INT(Browser,bottomline);
static FN_SET_INT(Browser,middleline);
static FN_GETSET_INT(Browser,column_char);
static FN_GETSET_INT(Browser,format_char);
static FN_GETSET_INT(Browser, value);
static FN_SET_INT(Browser,hide);
static FN_SET_INT(Browser,show);
static FN_GETSET_INT(Browser,position);
static FN_SET_INT2(Browser,move);
static FN_SET_STR(Browser,load);
static FN_GET_BOOL_SET_INT(Browser,selected);

static VALUE rb_FLBrowser_text(int argc, VALUE argv[], VALUE self){
  RBFLBrowser *widget = (RBFLBrowser*)rb_to_fl(self);
  VALUE arg1, arg2;
  char *str;
  switch( rb_scan_args(argc, argv, "11", &arg1, &arg2) ){
  case 1:
    str = (char*)(widget->text(NUM2INT(arg1)));
    return str ? rb_tainted_str_new2(str) : Qnil;
  case 2:
    str = (arg2 == Qnil) ? NULL : STR2CSTR(arg2);
    widget->text(NUM2INT(arg1), str);
    return Qnil;
  };
}

static VALUE rb_FLBrowser_column_widths(int argc, VALUE argv[], VALUE self)
{
  RBFLBrowser *widget = (RBFLBrowser*)rb_to_fl(self);
  VALUE ary;
  int *cary;

  switch( rb_scan_args(argc,argv,"01",&ary) ){
  case 0:
    cary = (int*) widget->column_widths();
    return cary2iary(cary);
  case 1:
    Check_Type(ary, T_ARRAY); /* must not be Qnil */
    cary = (int*) widget->column_widths();
    if( cary && (cary != no_columns) ){
      free(cary);
    };
    if( RARRAY(ary)->len == 0 ){
      widget->column_widths(no_columns);
    }
    else{
      widget->column_widths(iary2cary(ary));
    };
    return Qnil;
  };
}

static VALUE rb_FLBrowser_add(int argc, VALUE argv[], VALUE self)
{
  RBFLBrowser *widget = (RBFLBrowser*)rb_to_fl(self);
  VALUE str, data;

  switch( rb_scan_args(argc, argv, "11", &str, &data) ){
  case 1:
    widget->add(STR2CSTR(str));
    rb_ary_push(rb_ivar_get(self, rb_id_iitems), Qnil);
    break;
  case 2:
    widget->add(STR2CSTR(str), VAL2CPTR(data));
    rb_ary_push(rb_ivar_get(self, rb_id_iitems), data);
    break;
  };
  return Qnil;
}

static VALUE rb_FLBrowser_insert(int argc, VALUE argv[], VALUE self)
{
  RBFLBrowser *widget = (RBFLBrowser*)rb_to_fl(self);
  VALUE idx, str, data;

  switch( rb_scan_args(argc, argv, "21", &idx, &str, &data) ){
  case 2:
    widget->insert(NUM2INT(idx), STR2CSTR(str));
    rb_ary_insert(rb_ivar_get(self,rb_id_iitems), idx, Qnil);
    break;
  case 3:
    widget->insert(NUM2INT(idx), STR2CSTR(str), VAL2CPTR(data));
    rb_ary_insert(rb_ivar_get(self,rb_id_iitems), idx, data);
    break;
  };
  return Qnil;
}


static VALUE rb_FLBrowser_clear(VALUE self)
{
  RBFLBrowser *widget = (RBFLBrowser*)rb_to_fl(self);
  widget->clear();
  rb_ary_clear(rb_ivar_get(self, rb_id_iitems));
  return Qnil;
}

static VALUE rb_FLBrowser_data(int argc, VALUE argv[], VALUE self)
{
  RBFLBrowser *widget = (RBFLBrowser*)rb_to_fl(self);
  VALUE line, data;
  void *ret = 0;

  switch( rb_scan_args(argc, argv, "11", &line, &data) ){
  case 1:
    ret = widget->data(NUM2INT(line));
    break;
  case 2:
    widget->data(NUM2INT(line), VAL2CPTR(data));
    rb_ary_store(rb_ivar_get(self, rb_id_iitems), NUM2INT(line) - 1, data);
    break;
  };
  return CPTR2VAL(ret);
}

static VALUE rb_FLBrowser_remove(VALUE self, VALUE line)
{
  RBFLBrowser *widget = (RBFLBrowser*)rb_to_fl(self);
  widget->remove(NUM2INT(line));
  rb_ary_delete_at(rb_ivar_get(self, rb_id_iitems), NUM2INT(line) - 1);
  return Qnil;
}

static FN_S_NEW(Hold_Browser);
WIDGET_FNS(Hold_Browser);
static FN_GETSET_INT(Hold_Browser,value);

static FN_S_NEW(Multi_Browser);
WIDGET_FNS(Multi_Browser);

static FN_S_NEW(Select_Browser);
WIDGET_FNS(Select_Browser);

#ifdef HAVE_FL_FL_CHECK_BROWSER_H
static FN_S_NEW(Check_Browser);
WIDGET_FNS(Check_Browser);
static FN_GET_INT(Check_Browser, nitems);
static FN_GET_INT(Check_Browser, nchecked);
static FN_GET_BOOL_SET_INT(Check_Browser, checked);
static FN_SET_INT(Check_Browser, set_checked);
static FN_CALL(Check_Browser, check_all);
static FN_CALL(Check_Browser, check_none);
static FN_GET_INT(Check_Browser, value);
static FN_GET_STR_SET_INT(Check_Browser, text);
static FN_GET_BOOL_SET_STR_OPTBOOL(Check_Browser, add);
#endif

extern "C" void
Init_flbrowser(){
  rb_cFLBrowser_ = rb_define_class_under(rb_mFLTK, "Browser_", rb_cFLWidget);
  DEF_S_METHOD(Browser_,new,-1);
  DEF_WIDGET_METHODS(Browser_);
  DEF_METHOD(Browser_, selection, 0);
  DEF_METHOD(Browser_, select, -1);
  DEF_METHOD(Browser_, select_only, -1);
  DEF_METHOD(Browser_, deselect, 0);
  DEF_METHOD_EQ(Browser_, position, -1);
  DEF_METHOD(Browser_, has_scrollbar, 1);
  DEF_METHOD(Browser_, hposition, -1);
  DEF_METHOD(Browser_, incr_height, 0);
  DEF_METHOD(Browser_, item_quick_height, 1);
  DEF_METHOD_EQ(Browser_, textcolor, -1);
  DEF_METHOD_EQ(Browser_, textfont, -1);
  DEF_METHOD_EQ(Browser_, textsize, -1);
  DEF_METHOD(Browser_, item_select, -1);
  DEF_METHOD_Q(Browser_, item_selected, 1);
  DEF_METHOD(Browser_, find_item, 1);
  DEF_METHOD(Browser_, full_height, 0);
  DEF_METHOD(Browser_, full_width, 0);
  DEF_METHOD(Browser_, deleting, 1);
  DEF_METHOD(Browser_, inserting, 2);
  DEF_METHOD(Browser_, replacing, 2);
  DEF_METHOD(Browser_, new_list, 0);
  DEF_METHOD(Browser_, top, 0);
  DEF_METHOD(Browser_, redraw_line, 1);
  DEF_METHOD(Browser_, redraw_lines, 0);
  DEF_METHOD(Browser_, scrollbar_left, 0);
  DEF_METHOD(Browser_, scrollbar_right, 0);
  DEF_METHOD(Browser_, bbox, 0);
  DEF_METHOD(Browser_, leftedge, 0);

  DEF_MODULE_CONST_NUM(NORMAL_BROWSER);
  DEF_MODULE_CONST_NUM(SELECT_BROWSER);
  DEF_MODULE_CONST_NUM(HOLD_BROWSER);
  DEF_MODULE_CONST_NUM(MULTI_BROWSER);

  DEF_BROWSER_CONST_NUM(HORIZONTAL);
  DEF_BROWSER_CONST_NUM(VERTICAL);
  DEF_BROWSER_CONST_NUM(BOTH);
  DEF_BROWSER_CONST_NUM(HORIZONTAL_ALWAYS);
  DEF_BROWSER_CONST_NUM(VERTICAL_ALWAYS);
  DEF_BROWSER_CONST_NUM(BOTH_ALWAYS);
  DEF_BROWSER_CONST_NUM(ALWAYS_ON);

  rb_cFLBrowser  = rb_define_class_under(rb_mFLTK, "Browser", rb_cFLBrowser_);
  DEF_S_METHOD(Browser,new,-1);
  DEF_WIDGET_METHODS(Browser);
  DEF_INIT_METHOD(Browser);
  DEF_METHOD(Browser, clear, 0);
  DEF_METHOD(Browser, add, -1);
  DEF_METHOD(Browser, insert, -1);
  DEF_METHOD(Browser, remove, 1);
  DEF_METHOD(Browser, data, -1);
  DEF_METHOD(Browser, size, 0);
  DEF_METHOD(Browser, text, -1);
  DEF_METHOD(Browser, column_widths, -1);
  DEF_METHOD(Browser, column_char, -1);
  DEF_METHOD(Browser, format_char, -1);
  DEF_METHOD(Browser, bottomline, 1);
  DEF_METHOD(Browser, middleline, 1);
  DEF_METHOD(Browser, value, -1);
  DEF_METHOD(Browser, hide, 1);
  DEF_METHOD(Browser, show, 1);
  DEF_METHOD_EQ(Browser, position, -1);
  DEF_METHOD(Browser, move, 2);
  DEF_METHOD(Browser, load, 1);
  DEF_METHOD_Q(Browser, selected, 1);

  rb_cFLHold_Browser  = rb_define_class_under(rb_mFLTK, "HoldBrowser",
					      rb_cFLBrowser);
  DEF_S_METHOD(Hold_Browser,new,-1);
  DEF_WIDGET_METHODS(Hold_Browser);
  DEF_METHOD(Hold_Browser, value, -1);

  rb_cFLMulti_Browser  = rb_define_class_under(rb_mFLTK, "MultiBrowser",
					       rb_cFLBrowser);
  DEF_S_METHOD(Multi_Browser,new,-1);
  DEF_WIDGET_METHODS(Multi_Browser);

  rb_cFLSelect_Browser  = rb_define_class_under(rb_mFLTK, "SelectBrowser",
						rb_cFLBrowser);
  DEF_S_METHOD(Select_Browser,new,-1);
  DEF_WIDGET_METHODS(Select_Browser);

#ifdef HAVE_FL_FL_CHECK_BROWSER_H
  rb_cFLCheck_Browser = rb_define_class_under(rb_mFLTK, "CheckBrowser",
					      rb_cFLBrowser_);
  DEF_S_METHOD(Check_Browser,new,-1);
  DEF_WIDGET_METHODS(Check_Browser);
  DEF_METHOD(Check_Browser, nitems, 0);
  DEF_METHOD(Check_Browser, nchecked, 0);
  DEF_METHOD_EQ(Check_Browser, checked, 1);
  DEF_METHOD(Check_Browser, set_checked, 1);
  DEF_METHOD(Check_Browser, check_all, 0);
  DEF_METHOD(Check_Browser, check_none, 0);
  DEF_METHOD(Check_Browser, value, 0);
  DEF_METHOD(Check_Browser, text, 1);
  DEF_METHOD(Check_Browser, add, -1);
#endif
}
