/* -*- C++ -*-
 * $Id: flpixmap.cc,v 1.12 2006/12/29 05:07:32 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flwidget.h"
#include "flpixmap.h"

VALUE rb_cFLPixmap;
#if FLTK_VERSION >= FLTK_VERSION_CODE(1,1,0)
VALUE rb_cFLGIF_Image;
#endif

static void rb_FLPixmap_free(RBFLPixmap *pixmap)
{
};

static VALUE rb_FLPixmap_new(VALUE klass, RBFLPixmap *pixmap)
{
  VALUE obj;
  obj = Data_Wrap_Struct(klass, 0, rb_FLPixmap_free, pixmap);
  rb_fltk_objmap_aset((Fl_Widget*)pixmap, obj);
  return obj;
};

static VALUE rb_FLPixmap_initialize(int argc, VALUE argv[], VALUE self)
{
  return Qnil;
};

static VALUE rb_FLPixmap_s_new(int argc, VALUE argv[], VALUE klass)
{
  VALUE obj, xpm;
  RBFLPixmap *pixmap;
  char **data;
  char **rb_fltk_create_pixmap(VALUE);

  rb_scan_args(argc, argv, "10", &xpm);

  Check_Type(xpm, T_ARRAY);

  data = rb_fltk_create_pixmap(xpm);
  pixmap = new RBFLPixmap(data);
#if defined(HAVE_FL_PIXMAP_ALLOC_DATA)
  pixmap->alloc_data = 1;
#endif
  obj = rb_FLPixmap_new(klass,pixmap);

  rb_obj_call_init(obj, argc, argv);

  return obj;
};

static VALUE rb_FLPixmap_destroy(VALUE self)
{
  RBFLPixmap *pixmap = (RBFLPixmap*)rb_to_fl(self);

  rb_fltk_objmap_delete((Fl_Widget*)pixmap);

#if FLTK_VERSION >= FLTK_VERSION_CODE(1,1,0)
  if( pixmap ) delete pixmap;
#else
  if( pixmap && pixmap->data ){
    for( int i=0; pixmap->data[i]; i++ ){
      delete[] (pixmap->data[i]);
    };
    free((char**)(pixmap->data));
    delete pixmap;
  };
#endif

  return Qnil;
};

static VALUE rb_FLPixmap_label(VALUE self, VALUE vwidget)
{
  RBFLPixmap *pixmap = (RBFLPixmap*)rb_to_fl(self);
  RBFLWidget *widget = (RBFLWidget*)rb_to_fl(vwidget);

  pixmap->label(widget);

  return Qnil;
};

static VALUE rb_FLPixmap_draw(int argc, VALUE argv[], VALUE self)
{
  RBFLPixmap *pixmap = (RBFLPixmap*)rb_to_fl(self);
  VALUE arg1, arg2, arg3, arg4, arg5, arg6;
  int x, y, w, h, cx, cy;

  switch( rb_scan_args(argc, argv, "24", &arg1,&arg2,&arg3,&arg4,&arg5,&arg6) ){
  case 2:
    x = NUM2INT(arg1);
    y = NUM2INT(arg2);
    pixmap->draw(x,y);
    break;
  case 4:
    x = NUM2INT(arg1);
    y = NUM2INT(arg2);
    w = NUM2INT(arg3);
    h = NUM2INT(arg4);
    pixmap->draw(x,y,w,h);
    break;
  case 6:
    x = NUM2INT(arg1);
    y = NUM2INT(arg2);
    w = NUM2INT(arg3);
    h = NUM2INT(arg4);
    cx = NUM2INT(arg5);
    cy = NUM2INT(arg6);
    pixmap->draw(x,y,w,h,cx,cy);
    break;
  default:
    rb_raise(rb_eArgError, "too few/many arguments");
  };

  return Qnil;
};

#if FLTK_VERSION >= FLTK_VERSION_CODE(1,1,0)
static void rb_FLGIF_Image_free(RBFLPixmap *pixmap)
{
};

static VALUE rb_FLGIF_Image_new(VALUE klass, RBFLGIF_Image *img)
{
  VALUE obj;
  obj = Data_Wrap_Struct(klass, 0, rb_FLGIF_Image_free, img);
  rb_fltk_objmap_aset((Fl_Widget*)img, obj);
  return obj;
};

static VALUE rb_FLGIF_Image_initialize(int argc, VALUE argv[], VALUE self)
{
  return Qnil;
};

static VALUE rb_FLGIF_Image_s_new(int argc, VALUE argv[], VALUE klass)
{
  VALUE obj, filename;
  RBFLGIF_Image *img;

  rb_scan_args(argc, argv, "10", &filename);

  img = new RBFLGIF_Image(STR2CSTR(filename));
  obj = rb_FLGIF_Image_new(klass,img);

  rb_obj_call_init(obj, argc, argv);

  return obj;
};

static VALUE rb_FLGIF_Image_destroy(VALUE self)
{
  RBFLGIF_Image *img = (RBFLGIF_Image*)rb_to_fl(self);

  if( img ){
    rb_fltk_objmap_delete((Fl_Widget*)img);
    delete img;
  };
  return Qnil;
};
#endif

extern "C" void
Init_flpixmap(){
  rb_cFLPixmap = rb_define_class_under(rb_mFLTK, "Pixmap", rb_cObject);
  DEF_S_METHOD(Pixmap, new, -1);
  DEF_INIT_METHOD(Pixmap);
  DEF_METHOD(Pixmap, destroy, 0);
  DEF_METHOD(Pixmap, label, 1);
  DEF_METHOD(Pixmap, draw, -1);

#if FLTK_VERSION >= FLTK_VERSION_CODE(1,1,0)
  rb_cFLGIF_Image = rb_define_class_under(rb_mFLTK, "GIFImage", rb_cFLPixmap);
  DEF_S_METHOD(GIF_Image, new, -1);
  DEF_INIT_METHOD(GIF_Image);
  DEF_METHOD(GIF_Image, destroy, 0);
#endif
};
