# $Id: choice.rb,v 1.3 2006/12/31 11:58:23 jeremy_henty Exp $

# This program is a part of the Ruby/FLTK tests.
# Author:: Jeremy Henty <mailto:jeremy_henty@users.sourceforge.net>
# Copyright:: Copyright (c) 2006 Jeremy Henty 
# License:: LGPL

=begin

Demonstrate FLTK.choice (which passes its arguments to sprintf) and
FLTK.choice0 (which does not).

=end

require 'fltk'

stuff =
  [ [ :choice0, ], 
    [ :choice0, 
      "This is not a format, even though it contains a '%'!", 
      "Forget it", "No Way", "OK", ], 
    [ :choice0, 
      "Just one button", 
      "OK", ], 
    [ :choice0, 
      "Just two buttons", 
      "No Way", "OK", ], 
    [ :choice0, 
      "Different set of two buttons", 
      "Forget it", nil, "OK", ], 
    [ :choice0, 
      "Too many arguments!", 
      "Forget it", "No Way", "OK", "Maybe", ], 
    [ :choice, ],  
    [ :choice, 
      "This string is a format with an escaped '%%'!", 
      "Forget it", "No Way", "OK", ], 
    [ :choice, 
      "This format interpolates the string '%s' and the number %d.", 
      "Forget it", "No Way", "OK", 
      "Grand Wazoo", 42, ], 
    [ :choice, 
      "Formatted %s: %d buttons", 
      "Forget it", "No Way", nil, 
      "string", 2, ], 
    [ :choice, 
      "A '%S' makes this string a bad format, " +
      "so it will raise an error.", ], 
    [ :choice,
      "Too few format arguments (%s %s).", "hmm?", ], 
  ]

button_height = 30
win_height = stuff.size * button_height
FLTK::Window.new(900,win_height,"Choice"){|win|
  FLTK::Pack.new(win.w,1){|pack|
    pack.pack_type(FLTK::VERTICAL)
    stuff.each{|_m,*_a|
      label = "FLTK.%s(%s)" %
      [ _m, _a.collect{|_a0| _a0.inspect }.join(","), ]
      answers = _a[1..3]
      button = FLTK::Button.new(0,button_height,label){
        begin FLTK.message("Answer: %s",
                           answers[FLTK.send(_m,*_a)] || "None")
        rescue => err ; FLTK.alert("Error: %s",err)
        end
      }
      button.align(FLTK::ALIGN_INSIDE |
                   FLTK::ALIGN_LEFT   )
    }
  }
  win.show
}

FLTK.run
